# coding: utf-8

"""
ME TT DataPortal Data Portal API

ME TT DataPortal Data Portal APIs to fetch Gut Microbes Genomes / Genes information.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from mett_dataportal_sdk.models.gene_paginated_response_schema import (
    GenePaginatedResponseSchema,
)
from mett_dataportal_sdk.models.genome_paginated_response_schema import (
    GenomePaginatedResponseSchema,
)
from mett_dataportal_sdk.models.paginated_strain_drug_mic_response_schema import (
    PaginatedStrainDrugMICResponseSchema,
)
from mett_dataportal_sdk.models.paginated_strain_drug_metabolism_response_schema import (
    PaginatedStrainDrugMetabolismResponseSchema,
)
from mett_dataportal_sdk.models.strain_drug_data_response_schema import (
    StrainDrugDataResponseSchema,
)
from mett_dataportal_sdk.models.success_response_schema import SuccessResponseSchema

from mett_dataportal_sdk.api_client import ApiClient, RequestSerialized
from mett_dataportal_sdk.api_response import ApiResponse
from mett_dataportal_sdk.rest import RESTResponseType


class GenomesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def dataportal_api_core_genome_endpoints_get_all_genomes(
        self,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt], Field(description="Number of items per page.")
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GenomePaginatedResponseSchema:
        """Get all genomes

        Retrieves a paginated list of all genomes available in the system. Supports optional sorting by 'isolate_name' or 'species'.

        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of items per page.
        :type per_page: int
        :param sort_field: Field to sort by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_all_genomes_serialize(
            page=page,
            per_page=per_page,
            sort_field=sort_field,
            sort_order=sort_order,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenomePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_core_genome_endpoints_get_all_genomes_with_http_info(
        self,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt], Field(description="Number of items per page.")
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GenomePaginatedResponseSchema]:
        """Get all genomes

        Retrieves a paginated list of all genomes available in the system. Supports optional sorting by 'isolate_name' or 'species'.

        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of items per page.
        :type per_page: int
        :param sort_field: Field to sort by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_all_genomes_serialize(
            page=page,
            per_page=per_page,
            sort_field=sort_field,
            sort_order=sort_order,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenomePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_all_genomes_without_preload_content(
        self,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt], Field(description="Number of items per page.")
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all genomes

        Retrieves a paginated list of all genomes available in the system. Supports optional sorting by 'isolate_name' or 'species'.

        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of items per page.
        :type per_page: int
        :param sort_field: Field to sort by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_all_genomes_serialize(
            page=page,
            per_page=per_page,
            sort_field=sort_field,
            sort_order=sort_order,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenomePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_core_genome_endpoints_get_all_genomes_serialize(
        self,
        page,
        per_page,
        sort_field,
        sort_order,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if page is not None:
            _query_params.append(("page", page))

        if per_page is not None:
            _query_params.append(("per_page", per_page))

        if sort_field is not None:
            _query_params.append(("sortField", sort_field))

        if sort_order is not None:
            _query_params.append(("sortOrder", sort_order))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_essentiality_data_by_contig(
        self,
        isolate_name: Annotated[
            StrictStr, Field(description="Isolate name identifying the genome.")
        ],
        ref_name: Annotated[
            StrictStr,
            Field(
                description="Reference sequence (e.g. contig_1) name to retrieve essentiality data for."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SuccessResponseSchema:
        """Get essentiality data by genome and contig

        Retrieves cached essentiality data for a given genome isolate and reference name (e.g. contig_1). Returns gene essentiality information grouped by contig. This data is typically precomputed and used to visualize gene essentiality in genome browsers or analysis tools.

        :param isolate_name: Isolate name identifying the genome. (required)
        :type isolate_name: str
        :param ref_name: Reference sequence (e.g. contig_1) name to retrieve essentiality data for. (required)
        :type ref_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_essentiality_data_by_contig_serialize(
            isolate_name=isolate_name,
            ref_name=ref_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_core_genome_endpoints_get_essentiality_data_by_contig_with_http_info(
        self,
        isolate_name: Annotated[
            StrictStr, Field(description="Isolate name identifying the genome.")
        ],
        ref_name: Annotated[
            StrictStr,
            Field(
                description="Reference sequence (e.g. contig_1) name to retrieve essentiality data for."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SuccessResponseSchema]:
        """Get essentiality data by genome and contig

        Retrieves cached essentiality data for a given genome isolate and reference name (e.g. contig_1). Returns gene essentiality information grouped by contig. This data is typically precomputed and used to visualize gene essentiality in genome browsers or analysis tools.

        :param isolate_name: Isolate name identifying the genome. (required)
        :type isolate_name: str
        :param ref_name: Reference sequence (e.g. contig_1) name to retrieve essentiality data for. (required)
        :type ref_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_essentiality_data_by_contig_serialize(
            isolate_name=isolate_name,
            ref_name=ref_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_essentiality_data_by_contig_without_preload_content(
        self,
        isolate_name: Annotated[
            StrictStr, Field(description="Isolate name identifying the genome.")
        ],
        ref_name: Annotated[
            StrictStr,
            Field(
                description="Reference sequence (e.g. contig_1) name to retrieve essentiality data for."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get essentiality data by genome and contig

        Retrieves cached essentiality data for a given genome isolate and reference name (e.g. contig_1). Returns gene essentiality information grouped by contig. This data is typically precomputed and used to visualize gene essentiality in genome browsers or analysis tools.

        :param isolate_name: Isolate name identifying the genome. (required)
        :type isolate_name: str
        :param ref_name: Reference sequence (e.g. contig_1) name to retrieve essentiality data for. (required)
        :type ref_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_essentiality_data_by_contig_serialize(
            isolate_name=isolate_name,
            ref_name=ref_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_core_genome_endpoints_get_essentiality_data_by_contig_serialize(
        self,
        isolate_name,
        ref_name,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if isolate_name is not None:
            _path_params["isolate_name"] = isolate_name
        if ref_name is not None:
            _path_params["ref_name"] = ref_name
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/{isolate_name}/essentiality/{ref_name}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_genes_by_genome(
        self,
        isolate_name: Annotated[
            StrictStr, Field(description="Unique isolate name identifying the genome.")
        ],
        filter: Annotated[
            Optional[StrictStr],
            Field(
                description="Additional gene filter, e.g., 'pfam:pf07715;interpro:ipr012910'."
            ),
        ] = None,
        filter_operators: Annotated[
            Optional[StrictStr],
            Field(
                description="Logical operators (AND/OR) per facet, e.g., 'pfam:AND;interpro:OR'."
            ),
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt],
            Field(description="Number of genes to return per page."),
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort results by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GenePaginatedResponseSchema:
        """Get genes by genome isolate

        Retrieves a paginated list of genes associated with a specific genome isolate. Supports optional sorting by 'isolate_name', 'gene_name', 'alias', 'seq_id', 'locus_tag' and 'product'. Useful for viewing all genes within a selected genome.

        :param isolate_name: Unique isolate name identifying the genome. (required)
        :type isolate_name: str
        :param filter: Additional gene filter, e.g., 'pfam:pf07715;interpro:ipr012910'.
        :type filter: str
        :param filter_operators: Logical operators (AND/OR) per facet, e.g., 'pfam:AND;interpro:OR'.
        :type filter_operators: str
        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of genes to return per page.
        :type per_page: int
        :param sort_field: Field to sort results by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = (
            self._dataportal_api_core_genome_endpoints_get_genes_by_genome_serialize(
                isolate_name=isolate_name,
                filter=filter,
                filter_operators=filter_operators,
                page=page,
                per_page=per_page,
                sort_field=sort_field,
                sort_order=sort_order,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_core_genome_endpoints_get_genes_by_genome_with_http_info(
        self,
        isolate_name: Annotated[
            StrictStr, Field(description="Unique isolate name identifying the genome.")
        ],
        filter: Annotated[
            Optional[StrictStr],
            Field(
                description="Additional gene filter, e.g., 'pfam:pf07715;interpro:ipr012910'."
            ),
        ] = None,
        filter_operators: Annotated[
            Optional[StrictStr],
            Field(
                description="Logical operators (AND/OR) per facet, e.g., 'pfam:AND;interpro:OR'."
            ),
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt],
            Field(description="Number of genes to return per page."),
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort results by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GenePaginatedResponseSchema]:
        """Get genes by genome isolate

        Retrieves a paginated list of genes associated with a specific genome isolate. Supports optional sorting by 'isolate_name', 'gene_name', 'alias', 'seq_id', 'locus_tag' and 'product'. Useful for viewing all genes within a selected genome.

        :param isolate_name: Unique isolate name identifying the genome. (required)
        :type isolate_name: str
        :param filter: Additional gene filter, e.g., 'pfam:pf07715;interpro:ipr012910'.
        :type filter: str
        :param filter_operators: Logical operators (AND/OR) per facet, e.g., 'pfam:AND;interpro:OR'.
        :type filter_operators: str
        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of genes to return per page.
        :type per_page: int
        :param sort_field: Field to sort results by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = (
            self._dataportal_api_core_genome_endpoints_get_genes_by_genome_serialize(
                isolate_name=isolate_name,
                filter=filter,
                filter_operators=filter_operators,
                page=page,
                per_page=per_page,
                sort_field=sort_field,
                sort_order=sort_order,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_genes_by_genome_without_preload_content(
        self,
        isolate_name: Annotated[
            StrictStr, Field(description="Unique isolate name identifying the genome.")
        ],
        filter: Annotated[
            Optional[StrictStr],
            Field(
                description="Additional gene filter, e.g., 'pfam:pf07715;interpro:ipr012910'."
            ),
        ] = None,
        filter_operators: Annotated[
            Optional[StrictStr],
            Field(
                description="Logical operators (AND/OR) per facet, e.g., 'pfam:AND;interpro:OR'."
            ),
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt],
            Field(description="Number of genes to return per page."),
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort results by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get genes by genome isolate

        Retrieves a paginated list of genes associated with a specific genome isolate. Supports optional sorting by 'isolate_name', 'gene_name', 'alias', 'seq_id', 'locus_tag' and 'product'. Useful for viewing all genes within a selected genome.

        :param isolate_name: Unique isolate name identifying the genome. (required)
        :type isolate_name: str
        :param filter: Additional gene filter, e.g., 'pfam:pf07715;interpro:ipr012910'.
        :type filter: str
        :param filter_operators: Logical operators (AND/OR) per facet, e.g., 'pfam:AND;interpro:OR'.
        :type filter_operators: str
        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of genes to return per page.
        :type per_page: int
        :param sort_field: Field to sort results by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = (
            self._dataportal_api_core_genome_endpoints_get_genes_by_genome_serialize(
                isolate_name=isolate_name,
                filter=filter,
                filter_operators=filter_operators,
                page=page,
                per_page=per_page,
                sort_field=sort_field,
                sort_order=sort_order,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_core_genome_endpoints_get_genes_by_genome_serialize(
        self,
        isolate_name,
        filter,
        filter_operators,
        page,
        per_page,
        sort_field,
        sort_order,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if isolate_name is not None:
            _path_params["isolate_name"] = isolate_name
        # process the query parameters
        if filter is not None:
            _query_params.append(("filter", filter))

        if filter_operators is not None:
            _query_params.append(("filter_operators", filter_operators))

        if page is not None:
            _query_params.append(("page", page))

        if per_page is not None:
            _query_params.append(("per_page", per_page))

        if sort_field is not None:
            _query_params.append(("sort_field", sort_field))

        if sort_order is not None:
            _query_params.append(("sort_order", sort_order))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/{isolate_name}/genes",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_genomes_by_isolate_names(
        self,
        isolates: Annotated[
            StrictStr,
            Field(
                description="Comma-separated isolate names (e.g., 'BU_61,BU_909,BU_ATCC8492')."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SuccessResponseSchema:
        """Get genomes by isolate names

        Retrieves genome records for one or more isolate names. Accepts a comma-separated list of isolate names as input. Useful for batch lookups when isolate identifiers are known.

        :param isolates: Comma-separated isolate names (e.g., 'BU_61,BU_909,BU_ATCC8492'). (required)
        :type isolates: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_genomes_by_isolate_names_serialize(
            isolates=isolates,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_core_genome_endpoints_get_genomes_by_isolate_names_with_http_info(
        self,
        isolates: Annotated[
            StrictStr,
            Field(
                description="Comma-separated isolate names (e.g., 'BU_61,BU_909,BU_ATCC8492')."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SuccessResponseSchema]:
        """Get genomes by isolate names

        Retrieves genome records for one or more isolate names. Accepts a comma-separated list of isolate names as input. Useful for batch lookups when isolate identifiers are known.

        :param isolates: Comma-separated isolate names (e.g., 'BU_61,BU_909,BU_ATCC8492'). (required)
        :type isolates: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_genomes_by_isolate_names_serialize(
            isolates=isolates,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_genomes_by_isolate_names_without_preload_content(
        self,
        isolates: Annotated[
            StrictStr,
            Field(
                description="Comma-separated isolate names (e.g., 'BU_61,BU_909,BU_ATCC8492')."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get genomes by isolate names

        Retrieves genome records for one or more isolate names. Accepts a comma-separated list of isolate names as input. Useful for batch lookups when isolate identifiers are known.

        :param isolates: Comma-separated isolate names (e.g., 'BU_61,BU_909,BU_ATCC8492'). (required)
        :type isolates: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_genomes_by_isolate_names_serialize(
            isolates=isolates,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_core_genome_endpoints_get_genomes_by_isolate_names_serialize(
        self,
        isolates,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if isolates is not None:
            _query_params.append(("isolates", isolates))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/by-isolate-names",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_type_strains(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SuccessResponseSchema:
        """Get all type strains

        Returns a list of genomes that are designated as type strains. Type strains represent reference genomes for a given species and are essential for comparative analysis and classification.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_type_strains_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_core_genome_endpoints_get_type_strains_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SuccessResponseSchema]:
        """Get all type strains

        Returns a list of genomes that are designated as type strains. Type strains represent reference genomes for a given species and are essential for comparative analysis and classification.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_type_strains_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_get_type_strains_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all type strains

        Returns a list of genomes that are designated as type strains. Type strains represent reference genomes for a given species and are essential for comparative analysis and classification.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_get_type_strains_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SuccessResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_core_genome_endpoints_get_type_strains_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/type-strains",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_search_genomes_by_string(
        self,
        query: Annotated[
            Optional[StrictStr],
            Field(description="Search term to match against genome names or metadata."),
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt],
            Field(description="Number of genomes to return per page."),
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort results by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        isolates: Annotated[
            Optional[List[StrictStr]],
            Field(description="Optional list of isolate names to filter."),
        ] = None,
        species_acronym: Annotated[
            Optional[StrictStr],
            Field(description="Optional species acronym filter (BU, PV)."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GenomePaginatedResponseSchema:
        """Search genomes by query

        Searches genomes using a free-text query string. Returns a paginated list of matching genome records. Supports optional sorting by 'isolate_name' or 'species'.

        :param query: Search term to match against genome names or metadata.
        :type query: str
        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of genomes to return per page.
        :type per_page: int
        :param sort_field: Field to sort results by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param isolates: Optional list of isolate names to filter.
        :type isolates: List[str]
        :param species_acronym: Optional species acronym filter (BU, PV).
        :type species_acronym: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_search_genomes_by_string_serialize(
            query=query,
            page=page,
            per_page=per_page,
            sort_field=sort_field,
            sort_order=sort_order,
            isolates=isolates,
            species_acronym=species_acronym,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenomePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_core_genome_endpoints_search_genomes_by_string_with_http_info(
        self,
        query: Annotated[
            Optional[StrictStr],
            Field(description="Search term to match against genome names or metadata."),
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt],
            Field(description="Number of genomes to return per page."),
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort results by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        isolates: Annotated[
            Optional[List[StrictStr]],
            Field(description="Optional list of isolate names to filter."),
        ] = None,
        species_acronym: Annotated[
            Optional[StrictStr],
            Field(description="Optional species acronym filter (BU, PV)."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GenomePaginatedResponseSchema]:
        """Search genomes by query

        Searches genomes using a free-text query string. Returns a paginated list of matching genome records. Supports optional sorting by 'isolate_name' or 'species'.

        :param query: Search term to match against genome names or metadata.
        :type query: str
        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of genomes to return per page.
        :type per_page: int
        :param sort_field: Field to sort results by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param isolates: Optional list of isolate names to filter.
        :type isolates: List[str]
        :param species_acronym: Optional species acronym filter (BU, PV).
        :type species_acronym: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_search_genomes_by_string_serialize(
            query=query,
            page=page,
            per_page=per_page,
            sort_field=sort_field,
            sort_order=sort_order,
            isolates=isolates,
            species_acronym=species_acronym,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenomePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_core_genome_endpoints_search_genomes_by_string_without_preload_content(
        self,
        query: Annotated[
            Optional[StrictStr],
            Field(description="Search term to match against genome names or metadata."),
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="Page number to retrieve.")
        ] = None,
        per_page: Annotated[
            Optional[StrictInt],
            Field(description="Number of genomes to return per page."),
        ] = None,
        sort_field: Annotated[
            Optional[StrictStr], Field(description="Field to sort results by.")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="Sort order: 'asc' or 'desc'.")
        ] = None,
        isolates: Annotated[
            Optional[List[StrictStr]],
            Field(description="Optional list of isolate names to filter."),
        ] = None,
        species_acronym: Annotated[
            Optional[StrictStr],
            Field(description="Optional species acronym filter (BU, PV)."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Search genomes by query

        Searches genomes using a free-text query string. Returns a paginated list of matching genome records. Supports optional sorting by 'isolate_name' or 'species'.

        :param query: Search term to match against genome names or metadata.
        :type query: str
        :param page: Page number to retrieve.
        :type page: int
        :param per_page: Number of genomes to return per page.
        :type per_page: int
        :param sort_field: Field to sort results by.
        :type sort_field: str
        :param sort_order: Sort order: 'asc' or 'desc'.
        :type sort_order: str
        :param isolates: Optional list of isolate names to filter.
        :type isolates: List[str]
        :param species_acronym: Optional species acronym filter (BU, PV).
        :type species_acronym: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_core_genome_endpoints_search_genomes_by_string_serialize(
            query=query,
            page=page,
            per_page=per_page,
            sort_field=sort_field,
            sort_order=sort_order,
            isolates=isolates,
            species_acronym=species_acronym,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "GenomePaginatedResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_core_genome_endpoints_search_genomes_by_string_serialize(
        self,
        query,
        page,
        per_page,
        sort_field,
        sort_order,
        isolates,
        species_acronym,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {
            "isolates": "multi",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if query is not None:
            _query_params.append(("query", query))

        if page is not None:
            _query_params.append(("page", page))

        if per_page is not None:
            _query_params.append(("per_page", per_page))

        if sort_field is not None:
            _query_params.append(("sortField", sort_field))

        if sort_order is not None:
            _query_params.append(("sortOrder", sort_order))

        if isolates is not None:
            _query_params.append(("isolates", isolates))

        if species_acronym is not None:
            _query_params.append(("species_acronym", species_acronym))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/search",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_data(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> StrainDrugDataResponseSchema:
        """Get all drug data for a strain

        Retrieves both drug MIC and metabolism data for a specific strain. Returns comprehensive drug response information including resistance and metabolism patterns. No pagination is applied as this endpoint returns complete datasets for both MIC and metabolism data.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_data_serialize(
            isolate_name=isolate_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "StrainDrugDataResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_data_with_http_info(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[StrainDrugDataResponseSchema]:
        """Get all drug data for a strain

        Retrieves both drug MIC and metabolism data for a specific strain. Returns comprehensive drug response information including resistance and metabolism patterns. No pagination is applied as this endpoint returns complete datasets for both MIC and metabolism data.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_data_serialize(
            isolate_name=isolate_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "StrainDrugDataResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_data_without_preload_content(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all drug data for a strain

        Retrieves both drug MIC and metabolism data for a specific strain. Returns comprehensive drug response information including resistance and metabolism patterns. No pagination is applied as this endpoint returns complete datasets for both MIC and metabolism data.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_data_serialize(
            isolate_name=isolate_name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "StrainDrugDataResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_experimental_drug_endpoints_get_strain_drug_data_serialize(
        self,
        isolate_name,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if isolate_name is not None:
            _path_params["isolate_name"] = isolate_name
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["RoleBasedJWTAuth"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/{isolate_name}/drug-data",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_metabolism(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="Page number"),
        ] = None,
        per_page: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="Number of results per page"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PaginatedStrainDrugMetabolismResponseSchema:
        """Get drug metabolism data for a strain

        Retrieves drug metabolism data for a specific strain. Returns paginated degradation percentages, statistical significance, and metabolizer classifications.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param page: Page number
        :type page: int
        :param per_page: Number of results per page
        :type per_page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_metabolism_serialize(
            isolate_name=isolate_name,
            page=page,
            per_page=per_page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedStrainDrugMetabolismResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_metabolism_with_http_info(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="Page number"),
        ] = None,
        per_page: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="Number of results per page"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PaginatedStrainDrugMetabolismResponseSchema]:
        """Get drug metabolism data for a strain

        Retrieves drug metabolism data for a specific strain. Returns paginated degradation percentages, statistical significance, and metabolizer classifications.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param page: Page number
        :type page: int
        :param per_page: Number of results per page
        :type per_page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_metabolism_serialize(
            isolate_name=isolate_name,
            page=page,
            per_page=per_page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedStrainDrugMetabolismResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_metabolism_without_preload_content(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="Page number"),
        ] = None,
        per_page: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="Number of results per page"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get drug metabolism data for a strain

        Retrieves drug metabolism data for a specific strain. Returns paginated degradation percentages, statistical significance, and metabolizer classifications.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param page: Page number
        :type page: int
        :param per_page: Number of results per page
        :type per_page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_metabolism_serialize(
            isolate_name=isolate_name,
            page=page,
            per_page=per_page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedStrainDrugMetabolismResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_experimental_drug_endpoints_get_strain_drug_metabolism_serialize(
        self,
        isolate_name,
        page,
        per_page,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if isolate_name is not None:
            _path_params["isolate_name"] = isolate_name
        # process the query parameters
        if page is not None:
            _query_params.append(("page", page))

        if per_page is not None:
            _query_params.append(("per_page", per_page))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["RoleBasedJWTAuth"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/{isolate_name}/drug-metabolism",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_mic(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="Page number"),
        ] = None,
        per_page: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="Number of results per page"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PaginatedStrainDrugMICResponseSchema:
        """Get drug MIC data for a strain

        Retrieves drug MIC (Minimum Inhibitory Concentration) data for a specific strain. Returns paginated MIC measurements including drug names, values, units, and experimental conditions.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param page: Page number
        :type page: int
        :param per_page: Number of results per page
        :type per_page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_mic_serialize(
            isolate_name=isolate_name,
            page=page,
            per_page=per_page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedStrainDrugMICResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_mic_with_http_info(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="Page number"),
        ] = None,
        per_page: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="Number of results per page"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PaginatedStrainDrugMICResponseSchema]:
        """Get drug MIC data for a strain

        Retrieves drug MIC (Minimum Inhibitory Concentration) data for a specific strain. Returns paginated MIC measurements including drug names, values, units, and experimental conditions.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param page: Page number
        :type page: int
        :param per_page: Number of results per page
        :type per_page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_mic_serialize(
            isolate_name=isolate_name,
            page=page,
            per_page=per_page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedStrainDrugMICResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def dataportal_api_experimental_drug_endpoints_get_strain_drug_mic_without_preload_content(
        self,
        isolate_name: Annotated[StrictStr, Field(description="Strain isolate name")],
        page: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=1)]],
            Field(description="Page number"),
        ] = None,
        per_page: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(description="Number of results per page"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get drug MIC data for a strain

        Retrieves drug MIC (Minimum Inhibitory Concentration) data for a specific strain. Returns paginated MIC measurements including drug names, values, units, and experimental conditions.

        :param isolate_name: Strain isolate name (required)
        :type isolate_name: str
        :param page: Page number
        :type page: int
        :param per_page: Number of results per page
        :type per_page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._dataportal_api_experimental_drug_endpoints_get_strain_drug_mic_serialize(
            isolate_name=isolate_name,
            page=page,
            per_page=per_page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedStrainDrugMICResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _dataportal_api_experimental_drug_endpoints_get_strain_drug_mic_serialize(
        self,
        isolate_name,
        page,
        per_page,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if isolate_name is not None:
            _path_params["isolate_name"] = isolate_name
        # process the query parameters
        if page is not None:
            _query_params.append(("page", page))

        if per_page is not None:
            _query_params.append(("per_page", per_page))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["RoleBasedJWTAuth"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/genomes/{isolate_name}/drug-mic",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



"""Shared constants for the METT client ecosystem."""

# Production URL
DEFAULT_BASE_URL = "http://www.gut-microbes.org"

# For local development, set METT_BASE_URL environment variable:
# export METT_BASE_URL="http://localhost:8000"

__all__ = ["DEFAULT_BASE_URL"]



"""High-level METT Data Portal API client built on top of the generated SDK."""

from __future__ import annotations

import csv
from dataclasses import dataclass
from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    List,
    Mapping,
    Optional,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
)

import requests  # type: ignore[import]
from mett_dataportal_sdk import (
    ApiClient as SDKApiClient,
    Configuration as SDKConfiguration,
)
from mett_dataportal_sdk.api.drugs_api import DrugsApi
from mett_dataportal_sdk.api.essentiality_api import EssentialityApi
from mett_dataportal_sdk.api.fitness_api import FitnessApi
from mett_dataportal_sdk.api.genes_api import GenesApi
from mett_dataportal_sdk.api.genomes_api import GenomesApi
from mett_dataportal_sdk.api.mutant_growth_api import MutantGrowthApi
from mett_dataportal_sdk.api.pooled_ttp_interactions_api import PooledTTPInteractionsApi
from mett_dataportal_sdk.api.protein_protein_interactions_api import (
    ProteinProteinInteractionsApi,
)
from mett_dataportal_sdk.api.proteomics_api import ProteomicsApi
from mett_dataportal_sdk.api.reactions_api import ReactionsApi
from mett_dataportal_sdk.api.species_api import SpeciesApi
from mett_dataportal_sdk.exceptions import ApiException

from .config import Config, get_config
from .exceptions import APIError, AuthenticationError
from .request_utils import parse_tsv_response, request_json
from .models import (
    DrugMIC,
    DrugMetabolism,
    Gene,
    Genome,
    Pagination,
    Species,
)
from .utils import normalize_params, normalize_species_entry

T = TypeVar("T")
ApiType = TypeVar("ApiType")


@dataclass
class PaginatedResult(Generic[T]):
    items: List[T]
    pagination: Pagination | None
    raw: Dict[str, Any]


class DataPortalClient:
    """Thin wrapper that provides ergonomic helpers on top of the generated SDK."""

    def __init__(
        self,
        *,
        config: Config | None = None,
        base_url: str | None = None,
        jwt_token: str | None = None,
        timeout: int | None = None,
        verify_ssl: bool | None = None,
        user_agent: str | None = None,
        sdk_client: SDKApiClient | None = None,
    ) -> None:
        self.config = config or get_config()
        if base_url:
            self.config.base_url = base_url.rstrip("/")
        if jwt_token:
            self.config.jwt_token = jwt_token
        if timeout is not None:
            self.config.timeout = timeout
        if verify_ssl is not None:
            self.config.verify_ssl = verify_ssl
        if user_agent:
            self.config.user_agent = user_agent

        configuration = self._build_sdk_configuration()
        self._sdk_client = sdk_client or SDKApiClient(configuration=configuration)
        # align UA with the rest of the project
        self._sdk_client.user_agent = self.config.user_agent
        self._apis: Dict[Type[Any], Any] = {}
        self._http = self._build_http_session()

    # ------------------------------------------------------------------
    # Core API Methods
    # ------------------------------------------------------------------
    def list_species(self, *, format: str = "json") -> List[Species]:
        """List all species. Supports format='json' (default) or format='tsv'."""
        payload = request_json(
            self._http, self.config, "/api/species/", params={"format": format}
        )
        if isinstance(payload, dict):
            raw_items = payload.get("data") or []
        elif isinstance(payload, list):
            raw_items = payload
        else:
            raise APIError("Unexpected response for /api/species/")

        return [normalize_species_entry(item) for item in raw_items]

    def list_genomes(
        self, *, format: str = "json", **params: Any
    ) -> PaginatedResult[Genome]:
        """List all genomes. Supports format='json' (default) or format='tsv'."""
        if format == "tsv":
            return self._request_tsv_paginated(
                "/api/genomes/", params=params, model=Genome
            )
        response = self._call_api(
            self._api(GenomesApi).dataportal_api_core_genome_endpoints_get_all_genomes,
            params=params,
        )
        return self._to_paginated(response)

    def species_genomes(
        self, species_acronym: str, **params: Any
    ) -> PaginatedResult[Genome]:
        response = self._call_api(
            self._api(
                SpeciesApi
            ).dataportal_api_core_species_endpoints_get_genomes_by_species,
            params=params,
            species_acronym=species_acronym,
        )
        return self._to_paginated(response)

    def search_genomes(
        self, *, format: str = "json", **params: Any
    ) -> PaginatedResult[Genome]:
        """Search genomes. Supports format='json' (default) or format='tsv'."""
        if format == "tsv":
            return self._request_tsv_paginated(
                "/api/genomes/search", params=params, model=Genome
            )
        # Use direct HTTP request to ensure format=json is included in query string
        json_params = (params or {}).copy()
        json_params["format"] = "json"
        payload = request_json(
            self._http, self.config, "/api/genomes/search", params=json_params
        )
        # Parse paginated response
        if isinstance(payload, dict):
            data = payload.get("data", [])
            pagination_dict = payload.get("pagination")
            pagination = Pagination(**pagination_dict) if pagination_dict else None
            raw = payload
        else:
            data = payload if isinstance(payload, list) else []
            pagination = None
            raw = {"data": data}
        items = [Genome(**item) if isinstance(item, dict) else item for item in data]
        return PaginatedResult(items=items, pagination=pagination, raw=raw)

    def get_genome_genes(
        self, isolate_name: str, **params: Any
    ) -> PaginatedResult[Gene]:
        response = self._call_api(
            self._api(
                GenomesApi
            ).dataportal_api_core_genome_endpoints_get_genes_by_genome,
            params=params,
            isolate_name=isolate_name,
        )
        return self._to_paginated(response)

    def search_genes(self, **params: Any) -> PaginatedResult[Gene]:
        response = self._call_api(
            self._api(
                GenesApi
            ).dataportal_api_core_gene_endpoints_search_genes_by_string,
            params=params,
        )
        return self._to_paginated(response)

    def search_genes_advanced(self, **params: Any) -> PaginatedResult[Gene]:
        response = self._call_api(
            self._api(
                GenesApi
            ).dataportal_api_core_gene_endpoints_search_genes_by_multiple_genomes_and_species_and_string,
            params=params,
        )
        return self._to_paginated(response)

    def get_gene(self, locus_tag: str) -> Gene:
        response = self._call_api(
            self._api(
                GenesApi
            ).dataportal_api_core_gene_endpoints_get_gene_by_locus_tag,
            locus_tag=locus_tag,
        )
        return response

    # ------------------------------------------------------------------
    # Experimental API Methods
    # ------------------------------------------------------------------
    def search_drug_mic(
        self, *, format: str = "json", **params: Any
    ) -> PaginatedResult[DrugMIC]:
        """Search drug MIC data. Supports format='json' (default) or format='tsv'."""
        if format == "tsv":
            return self._request_tsv_paginated(
                "/api/drugs/mic/search", params=params, model=DrugMIC
            )
        response = self._call_api(
            self._api(
                DrugsApi
            ).dataportal_api_experimental_drug_endpoints_search_drug_mic,
            params=params,
        )
        return self._to_paginated(response)

    def search_drug_metabolism(self, **params: Any) -> PaginatedResult[DrugMetabolism]:
        response = self._call_api(
            self._api(
                DrugsApi
            ).dataportal_api_experimental_drug_endpoints_search_drug_metabolism,
            params=params,
        )
        return self._to_paginated(response)

    def get_strain_drug_mic(
        self, isolate_name: str, **params: Any
    ) -> PaginatedResult[DrugMIC]:
        response = self._call_api(
            self._api(
                GenomesApi
            ).dataportal_api_experimental_drug_endpoints_get_strain_drug_mic,
            params=params,
            isolate_name=isolate_name,
        )
        return self._to_paginated(response)

    def get_strain_drug_metabolism(
        self, isolate_name: str, **params: Any
    ) -> PaginatedResult[DrugMetabolism]:
        response = self._call_api(
            self._api(
                GenomesApi
            ).dataportal_api_experimental_drug_endpoints_get_strain_drug_metabolism,
            params=params,
            isolate_name=isolate_name,
        )
        return self._to_paginated(response)

    def get_strain_drug_data(self, isolate_name: str) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                GenomesApi
            ).dataportal_api_experimental_drug_endpoints_get_strain_drug_data,
            isolate_name=isolate_name,
        )
        return response.model_dump()

    def search_proteomics(self, **params: Any) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                ProteomicsApi
            ).dataportal_api_experimental_proteomics_endpoints_search_proteomics,
            params=params,
        )
        return response.model_dump()

    def search_essentiality(self, **params: Any) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                EssentialityApi
            ).dataportal_api_experimental_essentiality_endpoints_search_essentiality,
            params=params,
        )
        return response.model_dump()

    def search_fitness(self, **params: Any) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                FitnessApi
            ).dataportal_api_experimental_fitness_endpoints_search_fitness,
            params=params,
        )
        return response.model_dump()

    def search_mutant_growth(self, **params: Any) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                MutantGrowthApi
            ).dataportal_api_experimental_mutant_growth_endpoints_search_mutant_growth,
            params=params,
        )
        return response.model_dump()

    def search_reactions(self, **params: Any) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                ReactionsApi
            ).dataportal_api_experimental_reactions_endpoints_search_reactions,
            params=params,
        )
        return response.model_dump()

    # ------------------------------------------------------------------
    # Interactions API Methods
    # ------------------------------------------------------------------
    def search_ttp(self, **params: Any) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                PooledTTPInteractionsApi
            ).dataportal_api_interactions_ttp_endpoints_search_interactions,
            params=params,
        )
        return response.model_dump()

    def get_ttp_gene_interactions(
        self, locus_tag: str, **params: Any
    ) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                PooledTTPInteractionsApi
            ).dataportal_api_interactions_ttp_endpoints_get_gene_interactions,
            params=params,
            locus_tag=locus_tag,
        )
        return response.model_dump()

    def get_ttp_compound_interactions(
        self, compound: str, **params: Any
    ) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                PooledTTPInteractionsApi
            ).dataportal_api_interactions_ttp_endpoints_get_compound_interactions,
            params=params,
            compound=compound,
        )
        return response.model_dump()

    def search_ppi(self, **params: Any) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                ProteinProteinInteractionsApi
            ).dataportal_api_interactions_ppi_endpoints_search_ppi_interactions,
            params=params,
        )
        return response.model_dump()

    def get_ppi_neighbors(self, **params: Any) -> Dict[str, Any]:
        response = self._call_api(
            self._api(
                ProteinProteinInteractionsApi
            ).dataportal_api_interactions_ppi_endpoints_get_all_protein_neighbors,
            params=params,
        )
        return response.model_dump()

    # ------------------------------------------------------------------
    # Raw API Access
    # ------------------------------------------------------------------
    def raw_request(
        self,
        method: str,
        path: str,
        *,
        params: Optional[Union[Dict[str, Any], Sequence[Tuple[str, Any]]]] = None,
        headers: Optional[Mapping[str, str]] = None,
        data: Optional[Union[str, bytes]] = None,
        json_body: Optional[Any] = None,
        format: Optional[str] = None,
    ) -> requests.Response:
        """Low-level helper for issuing arbitrary API requests.

        Used by the CLI `mett api request` command to provide coverage for endpoints
        that do not have first-class helpers yet.
        """

        if json_body is not None and data is not None:
            raise ValueError("Provide only one of json_body or data")

        if path.startswith("http://") or path.startswith("https://"):
            url = path
        else:
            normalized = path if path.startswith("/") else f"/{path}"
            url = f"{self.config.base_url.rstrip('/')}{normalized}"

        request_headers = dict(headers or {})
        if format == "tsv":
            request_headers.setdefault("Accept", "text/tab-separated-values")
        elif format == "json" or format is None:
            request_headers.setdefault("Accept", "application/json")

        try:
            response = self._http.request(
                method=method.upper(),
                url=url,
                params=params,
                data=None if json_body is not None else data,
                json=json_body,
                headers=request_headers or None,
                timeout=self.config.timeout,
                verify=self.config.verify_ssl,
            )
        except requests.RequestException as exc:
            raise APIError(str(exc)) from exc

        try:
            response.raise_for_status()
        except requests.HTTPError as exc:
            message = response.text or str(exc)
            status = response.status_code if response is not None else None
            raise APIError(message, status_code=status) from exc

        return response

    # ------------------------------------------------------------------
    # Internal helpers
    # ------------------------------------------------------------------
    def _api(self, api_cls: Type[ApiType]) -> ApiType:
        if api_cls not in self._apis:
            self._apis[api_cls] = api_cls(self._sdk_client)
        return self._apis[api_cls]

    def _build_sdk_configuration(self) -> SDKConfiguration:
        configuration = SDKConfiguration(host=self.config.base_url.rstrip("/"))
        configuration.verify_ssl = self.config.verify_ssl
        token = self.config.jwt_token
        if token:
            configuration.access_token = token
        return configuration

    def _build_http_session(self) -> requests.Session:
        session = requests.Session()
        session.headers.update(
            {
                "Accept": "application/json",
                "User-Agent": self.config.user_agent,
            }
        )
        token = self.config.jwt_token
        if token:
            session.headers["Authorization"] = f"Bearer {token}"
        return session

    @property
    def _request_timeout(self) -> float:
        return float(self.config.timeout)

    def _call_api(
        self,
        func: Callable[..., T],
        *,
        params: Optional[Dict[str, Any]] = None,
        **kwargs: Any,
    ) -> T:
        normalized_params = normalize_params(params or {})
        normalized_params.update(kwargs)
        normalized_params["_request_timeout"] = self._request_timeout
        return self._call(func, **normalized_params)

    def _call(self, func: Callable[..., T], **kwargs: Any) -> T:
        try:
            return func(**kwargs)
        except ApiException as exc:
            if exc.status in {401, 403}:
                raise AuthenticationError(
                    exc.body or "Authentication failed", status_code=exc.status
                ) from exc
            message = exc.body or exc.reason or "API request failed"
            raise APIError(message, status_code=exc.status) from exc

    def _request_tsv_paginated(
        self,
        endpoint: str,
        *,
        params: Optional[Dict[str, Any]] = None,
        model: Type[T] | None = None,
    ) -> PaginatedResult[T]:
        """Make TSV request and parse into paginated result.

        Note: TSV responses may not include pagination metadata.
        If pagination info is missing, pagination will be None.
        """
        tsv_params = (params or {}).copy()
        tsv_params["format"] = "tsv"

        # Make direct HTTP request for TSV
        url = f"{self.config.base_url.rstrip('/')}{endpoint}"
        headers = {"Accept": "text/tab-separated-values"}
        token = self.config.jwt_token
        if token:
            headers["Authorization"] = f"Bearer {token}"

        try:
            resp = self._http.get(
                url,
                params=tsv_params,
                headers=headers,
                timeout=self.config.timeout,
                verify=self.config.verify_ssl,
            )
            resp.raise_for_status()

            # Parse TSV
            rows = parse_tsv_response(resp.text)

            # Convert to model instances if model provided
            items: List[T]
            if model is None:
                items = rows  # type: ignore[assignment]
            else:
                items = [model(**row) for row in rows]

            # TSV responses typically don't include pagination metadata
            # Check response headers or assume no pagination info
            pagination = None
            raw = {
                "data": [
                    dict(item) if hasattr(item, "model_dump") else item
                    for item in items
                ]
            }

            return PaginatedResult(items=items, pagination=pagination, raw=raw)
        except requests.exceptions.HTTPError as exc:
            if exc.response is not None and exc.response.status_code in {401, 403}:
                raise AuthenticationError(
                    "Authentication failed", status_code=exc.response.status_code
                ) from exc
            status = exc.response.status_code if exc.response is not None else None
            raise APIError(f"Request failed: {exc}", status_code=status) from exc
        except requests.exceptions.RequestException as exc:
            raise APIError(f"Request failed: {exc}") from exc
        except (ValueError, csv.Error) as exc:
            raise APIError(f"Failed to parse TSV response: {exc}") from exc

    @staticmethod
    def _to_paginated(schema: Any) -> PaginatedResult[Any]:
        data = list(schema.data or [])
        pagination = schema.pagination if hasattr(schema, "pagination") else None
        raw = schema.model_dump()
        return PaginatedResult(items=data, pagination=pagination, raw=raw)


__all__ = ["DataPortalClient", "PaginatedResult"]



from __future__ import annotations

from click.testing import CliRunner
from typer.main import get_command

from mett_client.cli.main import app as cli_app
from .test_cli import _patch_dummy_client


runner = CliRunner()
cli_cmd = get_command(cli_app)


def test_genomes_list(monkeypatch) -> None:
    """Friendly CLI: mett genomes list --page 1 --per-page 5 --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(
        cli_cmd,
        ["genomes", "list", "--page", "1", "--per-page", "5", "--format", "json"],
    )
    assert result.exit_code == 0


def test_genomes_search_query(monkeypatch) -> None:
    """Friendly CLI: mett genomes search --query PV_H4 --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(
        cli_cmd, ["genomes", "search", "--query", "PV_H4", "--format", "json"]
    )
    assert result.exit_code == 0


def test_genomes_search_iso_species_query(monkeypatch) -> None:
    """Friendly CLI: mett genomes search --query 909 --species BU --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(
        cli_cmd,
        ["genomes", "search", "--query", "909", "--species", "BU", "--format", "json"],
    )
    assert result.exit_code == 0


def test_genomes_type_strains(monkeypatch) -> None:
    """Friendly CLI: mett genomes type-strains --format json"""
    _patch_dummy_client(monkeypatch)
    result = runner.invoke(cli_cmd, ["genomes", "type-strains", "--format", "json"])
    assert result.exit_code == 0


def test_genomes_autocomplete(monkeypatch) -> None:
    """Friendly CLI: mett genomes autocomplete --query cc --limit 5 --format json"""
    _patch_dummy_client(monkeypatch)
    args = [
        "genomes",
        "autocomplete",
        "--query",
        "cc",
        "--limit",
        "5",
        "--format",
        "json",
    ]
    result = runner.invoke(cli_cmd, args)
    assert result.exit_code == 0


def test_genomes_by_isolates(monkeypatch) -> None:
    """Friendly CLI: mett genomes by-isolates --isolate BU_ATCC8492 --isolate PV_ATCC8482 --format json"""
    _patch_dummy_client(monkeypatch)
    args = [
        "genomes",
        "by-isolates",
        "--isolate",
        "BU_ATCC8492",
        "--isolate",
        "PV_ATCC8482",
        "--format",
        "json",
    ]
    result = runner.invoke(cli_cmd, args)
    assert result.exit_code == 0



"""Proteomics CLI commands."""

from __future__ import annotations

from typing import List, Optional

import typer  # type: ignore[import]

from ..utils import comma_join, ensure_client, handle_raw_response, merge_params

proteomics_app = typer.Typer(help="Proteomics endpoints")


@proteomics_app.command("search")
def proteomics_search(
    ctx: typer.Context,
    locus_tags: Optional[List[str]] = typer.Option(None, "--locus-tag"),
    uniprot_ids: Optional[List[str]] = typer.Option(None, "--uniprot"),
    min_coverage: Optional[float] = typer.Option(None, "--min-coverage"),
    min_unique_peptides: Optional[int] = typer.Option(None, "--min-unique-peptides"),
    has_evidence: Optional[bool] = typer.Option(None, "--has-evidence"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "locus_tags": comma_join(locus_tags),
            "uniprot_ids": comma_join(uniprot_ids),
            "min_coverage": min_coverage,
            "min_unique_peptides": min_unique_peptides,
            "has_evidence": has_evidence,
        }
    )
    response = client.raw_request(
        "GET", "/api/proteomics/search", params=params, format=format
    )
    handle_raw_response(response, format, title="Proteomics search")



"""Shared utility functions for CLI commands."""

from __future__ import annotations

from typing import Any, Dict, List, Optional, Sequence

import typer  # type: ignore[import]

from ..client import DataPortalClient
from ..config import get_config
from .output import print_full_table, print_json, print_tsv


def _build_client(
    *,
    base_url: Optional[str],
    jwt: Optional[str],
    timeout: Optional[int],
    verify_ssl: Optional[bool],
) -> DataPortalClient:
    """Build a DataPortalClient with the given configuration."""
    config = get_config()
    if base_url:
        config.base_url = base_url.rstrip("/")
    if jwt:
        config.jwt_token = jwt
    if timeout is not None:
        config.timeout = timeout
    if verify_ssl is not None:
        config.verify_ssl = verify_ssl
    return DataPortalClient(config=config)


def ensure_client(ctx: typer.Context) -> DataPortalClient:
    """Get or create a client from the Typer context."""
    if ctx.obj is None:
        ctx.obj = DataPortalClient(config=get_config())
    return ctx.obj


def parse_key_value_pairs(
    pairs: Optional[Sequence[str]],
    *,
    separator: str = "=",
    error_message: str = "Expected KEY=VALUE",
) -> Dict[str, str]:
    """Parse a sequence of KEY=VALUE strings into a dictionary."""
    parsed: Dict[str, str] = {}
    if not pairs:
        return parsed
    for pair in pairs:
        if separator not in pair:
            raise typer.BadParameter(error_message)
        key, value = pair.split(separator, 1)
        parsed[key.strip()] = value.strip()
    return parsed


def merge_params(*dicts: Dict[str, Any]) -> Dict[str, Any]:
    """Merge multiple dictionaries, skipping None values."""
    merged: Dict[str, Any] = {}
    for mapping in dicts:
        for key, value in mapping.items():
            if value is None:
                continue
            merged[key] = value
    return merged


def extract_table_rows(payload: Any) -> Optional[List[Any]]:
    """Extract table rows from a JSON payload."""
    if isinstance(payload, list):
        return payload
    if isinstance(payload, dict):
        for key in ("data", "results", "items"):
            value = payload.get(key)
            if isinstance(value, list):
                return value
    return None


def handle_raw_response(response: Any, format: Optional[str], *, title: str) -> None:
    """Handle a raw HTTP response and format it appropriately."""
    content_type = (response.headers.get("Content-Type") or "").lower()
    if (format or "").lower() == "tsv":
        # If TSV format requested, check if response is already TSV
        if "text/tab-separated" in content_type:
            typer.echo(response.text)
            return

        # Otherwise, parse JSON and convert to TSV
        try:
            payload = response.json()
        except ValueError:
            # If not JSON, just output as-is
            typer.echo(response.text)
            return

        # Extract rows from JSON payload
        rows = extract_table_rows(payload)
        if rows:
            print_tsv(rows)
        else:
            # If we can't extract rows, output as JSON (fallback)
            print_json(payload)
        return

    try:
        payload = response.json()
    except ValueError:
        typer.echo(response.text)
        return

    if (format or "").lower() == "json":
        print_json(payload)
        return

    rows = extract_table_rows(payload)
    if rows:
        print_full_table(rows, title=title)
    else:
        print_json(payload)


def comma_join(values: Optional[Sequence[str]]) -> Optional[str]:
    """Join a sequence of strings with commas."""
    if not values:
        return None
    return ",".join(values)


def print_paginated_result(result: Any, format: Optional[str], *, title: str) -> None:
    """Print a paginated result in the requested format."""
    if format == "tsv":
        print_tsv(result.items)
        return
    if format == "json":
        print_json(result.raw)
        return
    print_full_table(result.items, title=title)



"""Configuration helpers for the METT Data Portal client and CLI."""

from __future__ import annotations

from dataclasses import dataclass, field
import os
from pathlib import Path
from typing import Any, Dict

from .constants import DEFAULT_BASE_URL
from .exceptions import ConfigurationError
from .version import __version__

try:  # Python 3.11+
    import tomllib as tomli  # type: ignore[attr-defined]
except ModuleNotFoundError:  # pragma: no cover
    import tomli  # type: ignore[no-redef]


DEFAULT_TIMEOUT = 30
CONFIG_PATH = Path.home() / ".mett" / "config.toml"


@dataclass(slots=True)
class Config:
    """Runtime configuration for the DataPortal client."""

    base_url: str = DEFAULT_BASE_URL
    jwt_token: str | None = None
    timeout: int = DEFAULT_TIMEOUT
    verify_ssl: bool = True
    user_agent: str = field(default_factory=lambda: f"mett-client/{__version__}")

    @property
    def authorization_header(self) -> str | None:
        token = self.jwt_token
        return f"Bearer {token}" if token else None


def _load_from_file(path: Path) -> Dict[str, Any]:
    if not path.exists():
        return {}
    try:
        with path.open("rb") as fh:
            return tomli.load(fh)
    except (OSError, tomli.TOMLDecodeError) as exc:  # type: ignore[attr-defined]
        raise ConfigurationError(f"Unable to read config file {path}: {exc}") from exc


def _coerce_bool(value: Any | None) -> bool | None:
    if value is None:
        return None
    if isinstance(value, bool):
        return value
    if isinstance(value, str):
        lowered = value.strip().lower()
        if lowered in {"1", "true", "yes", "on"}:
            return True
        if lowered in {"0", "false", "no", "off"}:
            return False
    raise ConfigurationError(f"Cannot coerce value '{value}' to bool")


def get_config(
    *, config_path: Path | None = None, env: Dict[str, str] | None = None
) -> Config:
    """Load configuration from environment variables and optional TOML file."""

    env = env or os.environ
    path = config_path or CONFIG_PATH

    file_data = _load_from_file(path)
    cfg = Config()

    cfg.base_url = env.get("METT_BASE_URL") or file_data.get("base_url", cfg.base_url)
    cfg.jwt_token = env.get("METT_JWT") or file_data.get("jwt_token")

    timeout_val = env.get("METT_TIMEOUT") or file_data.get("timeout")
    if timeout_val:
        try:
            cfg.timeout = int(timeout_val)
        except (TypeError, ValueError) as exc:  # pragma: no cover
            raise ConfigurationError("METT_TIMEOUT must be an integer") from exc

    verify_val = env.get("METT_VERIFY_SSL") or file_data.get("verify_ssl")
    coerced = _coerce_bool(verify_val)
    if coerced is not None:
        cfg.verify_ssl = coerced

    cfg.user_agent = env.get("METT_USER_AGENT") or file_data.get(
        "user_agent", cfg.user_agent
    )

    return cfg


__all__ = ["Config", "get_config", "CONFIG_PATH"]



# coding: utf-8

"""
ME TT DataPortal Data Portal API

ME TT DataPortal Data Portal APIs to fetch Gut Microbes Genomes / Genes information.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import (
    BaseModel,
    ConfigDict,
    StrictBool,
    StrictFloat,
    StrictInt,
    StrictStr,
)
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Set
from typing_extensions import Self


class DrugMetabolismDataSchema(BaseModel):
    """
    Schema for individual drug metabolism measurement.
    """  # noqa: E501

    drug_name: Optional[StrictStr] = None
    drug_class: Optional[StrictStr] = None
    drug_subclass: Optional[StrictStr] = None
    compound_name: Optional[StrictStr] = None
    pubchem_id: Optional[StrictStr] = None
    degr_percent: Optional[Union[StrictFloat, StrictInt]] = None
    pval: Optional[Union[StrictFloat, StrictInt]] = None
    fdr: Optional[Union[StrictFloat, StrictInt]] = None
    metabolizer_classification: Optional[StrictStr] = None
    is_significant: Optional[StrictBool] = None
    experimental_condition_id: Optional[StrictInt] = None
    experimental_condition_name: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "drug_name",
        "drug_class",
        "drug_subclass",
        "compound_name",
        "pubchem_id",
        "degr_percent",
        "pval",
        "fdr",
        "metabolizer_classification",
        "is_significant",
        "experimental_condition_id",
        "experimental_condition_name",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DrugMetabolismDataSchema from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if drug_name (nullable) is None
        # and model_fields_set contains the field
        if self.drug_name is None and "drug_name" in self.model_fields_set:
            _dict["drug_name"] = None

        # set to None if drug_class (nullable) is None
        # and model_fields_set contains the field
        if self.drug_class is None and "drug_class" in self.model_fields_set:
            _dict["drug_class"] = None

        # set to None if drug_subclass (nullable) is None
        # and model_fields_set contains the field
        if self.drug_subclass is None and "drug_subclass" in self.model_fields_set:
            _dict["drug_subclass"] = None

        # set to None if compound_name (nullable) is None
        # and model_fields_set contains the field
        if self.compound_name is None and "compound_name" in self.model_fields_set:
            _dict["compound_name"] = None

        # set to None if pubchem_id (nullable) is None
        # and model_fields_set contains the field
        if self.pubchem_id is None and "pubchem_id" in self.model_fields_set:
            _dict["pubchem_id"] = None

        # set to None if degr_percent (nullable) is None
        # and model_fields_set contains the field
        if self.degr_percent is None and "degr_percent" in self.model_fields_set:
            _dict["degr_percent"] = None

        # set to None if pval (nullable) is None
        # and model_fields_set contains the field
        if self.pval is None and "pval" in self.model_fields_set:
            _dict["pval"] = None

        # set to None if fdr (nullable) is None
        # and model_fields_set contains the field
        if self.fdr is None and "fdr" in self.model_fields_set:
            _dict["fdr"] = None

        # set to None if metabolizer_classification (nullable) is None
        # and model_fields_set contains the field
        if (
            self.metabolizer_classification is None
            and "metabolizer_classification" in self.model_fields_set
        ):
            _dict["metabolizer_classification"] = None

        # set to None if is_significant (nullable) is None
        # and model_fields_set contains the field
        if self.is_significant is None and "is_significant" in self.model_fields_set:
            _dict["is_significant"] = None

        # set to None if experimental_condition_id (nullable) is None
        # and model_fields_set contains the field
        if (
            self.experimental_condition_id is None
            and "experimental_condition_id" in self.model_fields_set
        ):
            _dict["experimental_condition_id"] = None

        # set to None if experimental_condition_name (nullable) is None
        # and model_fields_set contains the field
        if (
            self.experimental_condition_name is None
            and "experimental_condition_name" in self.model_fields_set
        ):
            _dict["experimental_condition_name"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DrugMetabolismDataSchema from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "drug_name": obj.get("drug_name"),
                "drug_class": obj.get("drug_class"),
                "drug_subclass": obj.get("drug_subclass"),
                "compound_name": obj.get("compound_name"),
                "pubchem_id": obj.get("pubchem_id"),
                "degr_percent": obj.get("degr_percent"),
                "pval": obj.get("pval"),
                "fdr": obj.get("fdr"),
                "metabolizer_classification": obj.get("metabolizer_classification"),
                "is_significant": obj.get("is_significant"),
                "experimental_condition_id": obj.get("experimental_condition_id"),
                "experimental_condition_name": obj.get("experimental_condition_name"),
            }
        )
        return _obj



"""Experimental API CLI commands."""

from .drugs import drugs_app
from .essentiality import essentiality_app
from .fitness import fitness_app, fitness_corr_app
from .mutant_growth import mutant_app
from .operons import operons_app
from .orthologs import orthologs_app
from .proteomics import proteomics_app
from .reactions import reactions_app

__all__ = [
    "drugs_app",
    "proteomics_app",
    "essentiality_app",
    "fitness_app",
    "fitness_corr_app",
    "mutant_app",
    "reactions_app",
    "operons_app",
    "orthologs_app",
]



"""Other CLI commands (PyHMMER and raw API access)."""

from __future__ import annotations

import json
from pathlib import Path
from typing import Any, List, Optional

import typer  # type: ignore[import]

from .utils import (
    comma_join,
    ensure_client,
    handle_raw_response,
    merge_params,
    parse_key_value_pairs,
)

api_app = typer.Typer(help="Low-level raw API access")
pyhmmer_app = typer.Typer(help="PyHMMER endpoints")


def _load_body_json(body: Optional[str], body_file: Optional[Path]) -> Optional[Any]:
    if body and body_file:
        raise typer.BadParameter("Use either --body-json or --body-file, not both")
    if body:
        try:
            return json.loads(body)
        except json.JSONDecodeError as exc:
            raise typer.BadParameter(f"Invalid JSON payload: {exc}") from exc
    if body_file:
        return json.loads(body_file.read_text())
    return None


@api_app.command("request")
def api_request(
    ctx: typer.Context,
    method: str = typer.Argument(..., help="HTTP method (GET, POST, etc.)"),
    path: str = typer.Argument(..., help="API path, e.g. /api/species/"),
    format: Optional[str] = typer.Option(
        None, "--format", "-f", help="Response format: json|tsv"
    ),
    query: Optional[List[str]] = typer.Option(
        None, "--query", "-q", help="Query parameter KEY=VALUE"
    ),
    header: Optional[List[str]] = typer.Option(
        None, "--header", "-H", help="Extra header KEY:VALUE"
    ),
    data: Optional[str] = typer.Option(
        None, "--data", "-d", help="Raw request payload"
    ),
    json_body: Optional[str] = typer.Option(
        None, "--json", help="JSON request payload"
    ),
) -> None:
    """Invoke any METT API endpoint while reusing client configuration."""

    client = ensure_client(ctx)

    if data and json_body:
        raise typer.BadParameter("Use either --data or --json, not both")

    query_params = parse_key_value_pairs(query)
    header_params = parse_key_value_pairs(
        header,
        separator=":",
        error_message="Expected header format KEY:VALUE",
    )

    json_payload: Any = None
    if json_body:
        try:
            json_payload = json.loads(json_body)
        except json.JSONDecodeError as exc:
            raise typer.BadParameter(f"Invalid JSON payload: {exc}") from exc

    response = client.raw_request(
        method=method,
        path=path,
        params=query_params,
        headers=header_params,
        data=data,
        json_body=json_payload,
        format=format,
    )

    handle_raw_response(response, format, title=f"{method.upper()} {path}")


@pyhmmer_app.command("databases")
def pyhmmer_databases(
    ctx: typer.Context,
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request("GET", "/api/pyhmmer/search/databases", format=format)
    handle_raw_response(response, format, title="PyHMMER databases")


@pyhmmer_app.command("mx-choices")
def pyhmmer_mx_choices(
    ctx: typer.Context,
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET", "/api/pyhmmer/search/mx-choices", format=format
    )
    handle_raw_response(response, format, title="PyHMMER mx choices")


@pyhmmer_app.command("search")
def pyhmmer_search(
    ctx: typer.Context,
    body_json: Optional[str] = typer.Option(
        None, "--body-json", help="Inline JSON payload"
    ),
    body_file: Optional[Path] = typer.Option(
        None, "--body-file", exists=True, readable=True, help="Path to JSON body"
    ),
    format: Optional[str] = typer.Option("json", "--format", "-f", help="json|tsv"),
) -> None:
    client = ensure_client(ctx)
    payload = _load_body_json(body_json, body_file)
    if payload is None:
        raise typer.BadParameter("Provide --body-json or --body-file")
    response = client.raw_request(
        "POST",
        "/api/pyhmmer/search",
        json_body=payload,
        format=format,
    )
    handle_raw_response(response, format, title="PyHMMER search")


@pyhmmer_app.command("result")
def pyhmmer_result(
    ctx: typer.Context,
    job_id: str = typer.Argument(..., help="PyHMMER job ID"),
    page: Optional[int] = typer.Option(None, "--page", "-p"),
    page_size: Optional[int] = typer.Option(None, "--page-size"),
    taxonomy_ids: Optional[List[str]] = typer.Option(None, "--taxonomy-id"),
    architecture: Optional[str] = typer.Option(None, "--architecture"),
    with_domains: Optional[bool] = typer.Option(None, "--with-domains"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = merge_params(
        {
            "page": page,
            "page_size": page_size,
            "taxonomy_ids": comma_join(taxonomy_ids),
            "architecture": architecture,
            "with_domains": with_domains,
        }
    )
    response = client.raw_request(
        "GET",
        f"/api/pyhmmer/result/{job_id}",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"PyHMMER result ({job_id})")


@pyhmmer_app.command("domains")
def pyhmmer_domains(
    ctx: typer.Context,
    job_id: str = typer.Argument(...),
    target: str = typer.Option(..., "--target"),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    params = {"target": target}
    response = client.raw_request(
        "GET",
        f"/api/pyhmmer/result/{job_id}/domains",
        params=params,
        format=format,
    )
    handle_raw_response(response, format, title=f"PyHMMER domains ({job_id})")


@pyhmmer_app.command("download")
def pyhmmer_download(
    ctx: typer.Context,
    job_id: str = typer.Argument(...),
    download_format: str = typer.Option(
        ..., "--download-format", help="aligned_fasta|fasta|csv|tab"
    ),
    output: Optional[Path] = typer.Option(None, "--output", "-o"),
) -> None:
    client = ensure_client(ctx)
    params = {"format": download_format}
    response = client.raw_request(
        "GET",
        f"/api/pyhmmer/result/{job_id}/download",
        params=params,
        format="tsv" if download_format == "tab" else None,
    )
    content = response.text
    if output:
        output.write_text(content)
        typer.echo(f"Wrote {output}")
    else:
        typer.echo(content)


@pyhmmer_app.command("debug-msa")
def pyhmmer_debug_msa(
    ctx: typer.Context,
    job_id: str = typer.Argument(...),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET",
        f"/api/pyhmmer/result/{job_id}/debug-pyhmmer-msa",
        format=format,
    )
    handle_raw_response(response, format, title=f"PyHMMER MSA ({job_id})")


@pyhmmer_app.command("debug-fasta")
def pyhmmer_debug_fasta(
    ctx: typer.Context,
    job_id: str = typer.Argument(...),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET",
        f"/api/pyhmmer/result/{job_id}/debug-fasta",
        format=format,
    )
    handle_raw_response(response, format, title=f"PyHMMER FASTA ({job_id})")


@pyhmmer_app.command("debug-task")
def pyhmmer_debug_task(
    ctx: typer.Context,
    task_id: str = typer.Argument(...),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    response = client.raw_request(
        "GET", f"/api/pyhmmer/debug/task/{task_id}", format=format
    )
    handle_raw_response(response, format, title=f"PyHMMER task ({task_id})")


@pyhmmer_app.command("testtask")
def pyhmmer_testtask(
    ctx: typer.Context,
    body_json: Optional[str] = typer.Option(None, "--body-json"),
    body_file: Optional[Path] = typer.Option(
        None, "--body-file", exists=True, readable=True
    ),
    format: Optional[str] = typer.Option(None, "--format", "-f"),
) -> None:
    client = ensure_client(ctx)
    payload = _load_body_json(body_json, body_file)
    response = client.raw_request(
        "POST", "/api/pyhmmer/testtask", json_body=payload, format=format
    )
    handle_raw_response(response, format, title="PyHMMER test task")
